searchNodes=[{"doc":"Functions for dealing with machines: to wit, generate/2, which creates an initial %State{} struct from a %Machine{}, and advance/3 (along with companion advance!/3) which transforms one %State{} into another one according to the rules given in the machine.","ref":"Maxine.html","title":"Maxine","type":"module"},{"doc":"Creates a new state record based on the current state, an event, and options (optionally). Parameterscurrent: A %State{} record; the current stateevent: The name of the event you'd like to call (atom)options: optional arguments to the event; will be preserved in state.data.optionsExamplesiex&gt; alias Maxine.Examples.Package iex&gt; state = Maxine.generate(Package.machine) iex&gt; {:ok, state = %Maxine.State{}} = Maxine.advance(state, :ship) iex&gt; state.name :in_transit iex&gt; alias Maxine.Examples.Package iex&gt; state = Maxine.generate(Package.machine) iex&gt; {:ok, state = %Maxine.State{}} = Maxine.advance(state, :ship, foo: &quot;bar&quot;) iex&gt; state.data.options[:foo] &quot;bar&quot;","ref":"Maxine.html#advance/3","title":"Maxine.advance/3","type":"function"},{"doc":"Exception-raising wrapper (unwrapper?) for the advance/3.Parameterscurrent: A %State{} record; the current stateevent: The name of the event you'd like to call (atom)options: optional arguments to the event; will be preservedExamplesiex&gt; alias Maxine.Examples.Package iex&gt; state = Maxine.generate(Package.machine) iex&gt; state = Maxine.advance!(state, :ship) iex&gt; state.name :in_transit","ref":"Maxine.html#advance!/3","title":"Maxine.advance!/3","type":"function"},{"doc":"Create a new state machine, optionally specifying a starting state other than the one given in the machine.Parametersmachine: An instance of %Machine{} that specifies events, transitions, states, callbacksinitial: An optional initial state, overriding the machine's defaultExamplesiex&gt; alias Maxine.Examples.Package iex&gt; Maxine.generate(Package.machine).name :origin iex&gt; alias Maxine.Examples.Package iex&gt; Maxine.generate(Package.machine, :foobar).name :foobar","ref":"Maxine.html#generate/2","title":"Maxine.generate/2","type":"function"},{"doc":"Helper functions for use in callbacks. These get their own module because they really don't belong with generate/2 and advance/3. The idea is that you import Maxinewhen you're controlling state machines, and you import Maxine.Callbackswhen you're writing callback functions.","ref":"Maxine.Callbacks.html","title":"Maxine.Callbacks","type":"module"},{"doc":"Avoid annoying one-layer &quot;deep merge&quot; issues when sticking things in %Data{} structs.Parametersdata: The %Data{} struct we're merging intosection: The slice of data we're using; can be :app, :options or :tmpto_merge: A map holding the data we want to mergeExamplesiex&gt; new_data = Maxine.Callbacks.merge_data(%Maxine.Data{}, :app, %{hello: &quot;world&quot;}) iex&gt; new_data.app[:hello] &quot;world&quot;","ref":"Maxine.Callbacks.html#merge_data/3","title":"Maxine.Callbacks.merge_data/3","type":"function"},{"doc":"Tag a %Data{} with an event to fire automatically next. (Note here: Checking the key on the tmp map directly as we do in the examples below is not how you want to use it, and means implementation is leaking into the doctests, which isn't ideal, but the corresponding function to read this stuff is private in Maxine so it's frankly easier and more transparent here to just roll with it, as long as we're all clear that you should probably not do things like this in production/real life.)Parametersdata: the %Data{} struct we're tagging - event: the event we'd like fired when this callback cycle is done (atom) - options: Optional, any thing we'd like to go along with the eventExamplesiex&gt; new_data = Maxine.Callbacks.request(%Maxine.Data{}, :ship, foo: &quot;bar&quot;) iex&gt; new_data.tmp[:_maxine_next_event] :ship iex&gt; new_data = Maxine.Callbacks.request(%Maxine.Data{}, :ship, foo: &quot;bar&quot;) iex&gt; new_data.tmp[:_maxine_next_options][:foo] &quot;bar&quot;","ref":"Maxine.Callbacks.html#request/3","title":"Maxine.Callbacks.request/3","type":"function"},{"doc":"Compose individual machines using the optional DeepMerge dependency.","ref":"Maxine.Compose.html","title":"Maxine.Compose","type":"module"},{"doc":"Compose machines.","ref":"Maxine.Compose.html#compose/1","title":"Maxine.Compose.compose/1","type":"function"},{"doc":"Where we put application data; the only way for callbacks to communicate with the outside world. Note that at each event, :tmp is wiped and :options is updated to reflect whatever the current event was called with.","ref":"Maxine.Data.html","title":"Maxine.Data","type":"module"},{"doc":"","ref":"Maxine.Data.html#t:sections/0","title":"Maxine.Data.sections/0","type":"type"},{"doc":"","ref":"Maxine.Data.html#t:t/0","title":"Maxine.Data.t/0","type":"type"},{"doc":"Ecto bindings for Maxine. Provides:cast_state/4 for updating state on a changeset","ref":"Maxine.Ecto.html","title":"Maxine.Ecto","type":"module"},{"doc":"Changeset helper that uses the current state of the underlying record along with the passed in event to determine what the next state should be, according to the passed in machine. If the change is successful, the new state is written to the field. If not, an error is added to the changeset.Options:field - the field in which the changeset keeps its state; default is :stateOptions will be passed to the underlying call to advance/3 as event options.Note that it does not look for the relevant state in the params, because we assume cast_state/4 here is the only thing that will ever update that field. Plan accordingly.","ref":"Maxine.Ecto.html#cast_state/4","title":"Maxine.Ecto.cast_state/4","type":"function"},{"doc":"The various types of failure we may encounter.","ref":"Maxine.Errors.html","title":"Maxine.Errors","type":"module"},{"doc":"","ref":"Maxine.Errors.html#t:error/0","title":"Maxine.Errors.error/0","type":"type"},{"doc":"The actual machine structure","ref":"Maxine.Machine.html","title":"Maxine.Machine","type":"module"},{"doc":"","ref":"Maxine.Machine.html#t:callback/0","title":"Maxine.Machine.callback/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:callback_map/0","title":"Maxine.Machine.callback_map/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:cb_index/0","title":"Maxine.Machine.cb_index/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:cb_listing/0","title":"Maxine.Machine.cb_listing/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:cb_name/0","title":"Maxine.Machine.cb_name/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:event_name/0","title":"Maxine.Machine.event_name/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:event_options/0","title":"Maxine.Machine.event_options/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:groups_map/0","title":"Maxine.Machine.groups_map/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:name/0","title":"Maxine.Machine.name/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:state_name/0","title":"Maxine.Machine.state_name/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:t/0","title":"Maxine.Machine.t/0","type":"type"},{"doc":"","ref":"Maxine.Machine.html#t:transition_map/0","title":"Maxine.Machine.transition_map/0","type":"type"},{"doc":"Sentinel values used by Maxine to signal that no further event has been requested, (Halt) and that no transition for the current state was found for a given name of that state, and resolve_next_state should keep looking under the state's other names (Pass).","ref":"Maxine.Signals.html","title":"Maxine.Signals","type":"module"},{"doc":"","ref":"Maxine.Signals.Halt.html","title":"Maxine.Signals.Halt","type":"module"},{"doc":"","ref":"Maxine.Signals.Pass.html","title":"Maxine.Signals.Pass","type":"module"},{"doc":"The &quot;state&quot; part of &quot;state machine.&quot; These are the structs we actually pass around, call events on, and transform in the course of normal use.","ref":"Maxine.State.html","title":"Maxine.State","type":"module"},{"doc":"","ref":"Maxine.State.html#t:t/0","title":"Maxine.State.t/0","type":"type"},{"doc":"For failures in the callback phase. For use by callbacks themselves; we don't call this one directly here. Use cause to wrap a more specific error.","ref":"Maxine.Errors.CallbackError.html","title":"Maxine.Errors.CallbackError","type":"exception"},{"doc":"When a machine returns something other than a %Data{} or a %CallbackError.","ref":"Maxine.Errors.CallbackReturnError.html","title":"Maxine.Errors.CallbackReturnError","type":"exception"},{"doc":"Called when the underlying machine is faulty and causes something unhandleable in the API.","ref":"Maxine.Errors.MachineError.html","title":"Maxine.Errors.MachineError","type":"exception"},{"doc":"When a machine requests a non-existent callback function.","ref":"Maxine.Errors.NoSuchCallbackError.html","title":"Maxine.Errors.NoSuchCallbackError","type":"exception"},{"doc":"On a non-existent event.","ref":"Maxine.Errors.NoSuchEventError.html","title":"Maxine.Errors.NoSuchEventError","type":"exception"},{"doc":"When the event has no transition mapped for the present state.","ref":"Maxine.Errors.UnavailableEventError.html","title":"Maxine.Errors.UnavailableEventError","type":"exception"}]